

#include "LPC17xx.h"
#include <string.h>
#include <stdio.h>
#include "uart.h"
#include "printf.h"
#include "driverWifly.h"
#include "utilsLPC17xx.h"
#include "GPIO.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"


/* Declare a variable of type xSemaphoreHandle */
xSemaphoreHandle xMutex;


/*****************************************************************************
** Function name:		wiflyConnect
**
** Descriptions:		Connects to Wifly module and enters command mode
**
** parameters:			None
** Returned value:		1 if connection success, 0 if failed
**
*****************************************************************************/

uint8_t wiflyConnect(){

	char respostaWifly[BUFSIZE];
	uint8_t connectat = 0;

	/* Active delay to wait for Wifly to be totally initialized */

	delayMs ( 1000 );


	//Initialize pin
	//LPC_PINCON->PINSEL1 &= ( ~( 3 << 12 ) );
	//LPC_GPIO0->FIODIR |= ( 1 << RST_WIFLY_PIN );

	GPIOSet ( RST_WIFLY_PIN , 1 );

	// reset the Wifly
	//LPC_GPIO0->FIOCLR = 1 << RST_WIFLY_PIN;
	//delayMs ( 1 ); // min 160us
	//LPC_GPIO0->FIOSET = 1 << RST_WIFLY_PIN;

	GPIOWrite ( RST_WIFLY_PIN , 0 );
	delayMs ( 1 );
	GPIOWrite ( RST_WIFLY_PIN , 1 );

	/* We receive the answer from Wifly after reset */

	UARTScan ( respostaWifly , 4000 );

	/* We enter in comand mode */

	printText ( 0 , "$$$" );

	/* We receive the answer from Wifly after reset */

	UARTScan ( respostaWifly , 4000 );

	/* We check answer received from Wifly */

	connectat = parseString ( respostaWifly , MSG_CMD );

	/* We create a mutex type semaphor to avoid concurrent access to printf */
	xMutex = xSemaphoreCreateMutex();

	return connectat;

}


/*****************************************************************************
** Function name:		wiflyNetAccess
**
** Descriptions:		Connects Wifly to the required network
**
** parameters:			SSID, authentication mode, password
** Returned value:		1 if connection success, 0 if fails
**
*****************************************************************************/

uint8_t wiflyNetAccess(char *ssid, char *mode, char *pswd){

	char texte[BUFSIZE];
	char respostaWifly[BUFSIZE];
	uint8_t connectat = 0;


	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */

	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	/* We set the authentication mode */

	if ( strstr ( mode , "wpa" ) !=0 ){
		sprintf ( texte , "set wlan phrase %s\r\n" , pswd );
		}

	if ( strstr ( mode , "wep" ) !=0 ){
		sprintf ( texte , "set wlan key %s\n" , pswd );
		}

	printText ( 0 , texte );

	/* We receive the answer from Wifly */

	UARTScan ( respostaWifly , 4000 );

	/* we join the network */

	sprintf ( texte , "join %s\r\n" , ssid );
	printText ( 0 , texte );
	UARTScan ( respostaWifly , 4000 );

	/* we check the answer from wifly */

	if ( parseString ( respostaWifly , MSG_CNX ) !=0 ){
		connectat = 1;
	}

	}

	xSemaphoreGive ( xMutex );

	return connectat;

}

/*****************************************************************************
** Function name:		wiflyPing
**
** Descriptions:		Pings the specified address
**
** parameters:			address to ping
** Returned value:		1 if connection success, 0 if fails
**
*****************************************************************************/

uint8_t wiflyPing ( char *address ){

	char texte[BUFSIZE];
	char respostaWifly[BUFSIZE];
	uint8_t correcte = 0;

	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */

	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	sprintf ( texte , "ping %s\r\n" , address );

	/* We send the command to Wifly */

	printText ( 0 , texte );

	/* We receive the answer from Wifly */
	UARTScan ( respostaWifly , 10000 );

	/* we check the answer from wifly */

	if ( parseString ( respostaWifly , MSG_PING ) != 0 ){
		correcte = 1;
	}

	}

	xSemaphoreGive ( xMutex );

	return correcte;
}


/*****************************************************************************
** Function name:		wiflyGetIP
**
** Descriptions:		Get the IP of Wifly
**
** parameters:			string to store returned value
** Returned value:		1 if command correct, 0 if failed
**
*****************************************************************************/

uint8_t wiflyGetIP ( char *resposta ){

	char respostaWifly[BUFSIZE];
	char *reply;
	uint32_t i;

	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */

	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	/* We send the command to Wifly */
	printText ( 0 , "get ip\r\n" );

	/* We receive the answer from Wifly */
	UARTScan ( respostaWifly , 4000 );

	/* we check the answer from wifly */

	i = parseString ( respostaWifly , "IP=" );

	if ( i != 0 ){

		reply = &( respostaWifly[i] );
		cutString ( reply , ':' );
		strcpy ( resposta , reply );

		i = 1;
	}

	}

	xSemaphoreGive ( xMutex );

	return i;

}

/*****************************************************************************
** Function name:		wiflyGetChannel
**
** Descriptions:		Get the channel where wiFly is connected
**
** parameters:			string to store returned value
** Returned value:		1 if command correct, 0 if failed
**
*****************************************************************************/

uint8_t wiflyGetChannel( char *resposta ){

	char respostaWifly[BUFSIZE];
	char *reply;
	uint32_t i;

	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */

	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	/* We send the command to Wifly */
	printText ( 0 , "show net\r\n" );

	/* We receive the answer from Wifly */
	UARTScan ( respostaWifly , 4000 );

	/* we check the answer from wifly */

	i= parseString ( respostaWifly , "Chan=" );

	if ( i != 0 ){

		reply =  &( respostaWifly[i] );
		cutString( reply , '\r' );
		strcpy ( resposta , reply );

		i = 1;
	}
	}

	xSemaphoreGive ( xMutex );

	return i;

}

/*****************************************************************************
** Function name:		wiflyGetRSSI
**
** Descriptions:		Get the wifi signal strength
**
** parameters:			none
** Returned value:		signal strength, 0 if failed
**
*****************************************************************************/

signed int wiflyGetRSSI (){

	char respostaWifly[BUFSIZE];
	char *reply;
	char string[10];
	uint32_t i;
	signed int value = 0;


	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */
	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	/* We send the command to Wifly */
	printText( 0 , "show rssi\r\n" );

	/* We receive the answer from Wifly */
	UARTScan ( respostaWifly , 4000 );

	/* we check the answer from wifly */

	i= parseString ( respostaWifly , "RSSI=(" );

	if ( i != 0 ){

			reply = &( respostaWifly[i] );
			cutString ( reply , ')' );

			strcpy ( string , reply);

			if ( strlen ( reply ) == 2 ) value = - ( string[1] - '0' );

			if ( strlen ( reply ) == 3 )value = - ( ( (10* ( string[1] - '0' ))  + ( string[2] - '0' ) ) );

	}

	}

	xSemaphoreGive ( xMutex );

	return value;

}

/*****************************************************************************
** Function name:		wiflyGetTime
**
** Descriptions:		Get the time since module powered on
**
** parameters:			string to store returned value
** Returned value:		1 if command correct, 0 if failed
**
*****************************************************************************/

uint8_t wiflyGetTime ( char *resposta ){

	char respostaWifly[BUFSIZE];
	char *reply;
	uint32_t i;

	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */
	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	/* We send the command to Wifly */
	printText ( 0 , "show time\r\n" );

	/* We receive the answer from Wifly */
	UARTScan ( respostaWifly , 4000 );

	/* We check the answer from wifly */

	i = parseString ( respostaWifly , "UpTime=" );

	if ( i != 0 ){

			reply = &( respostaWifly[i] );
			cutString ( reply , '\r' );
			strcpy ( resposta , reply  );

			i = 1;
	}
	}

	xSemaphoreGive ( xMutex );

	return i;

}

/*****************************************************************************
** Function name:		wiflySendHTTP
**
** Descriptions:		Send a comand by HTTP protocol
**
** parameters:			Comand to be send, string to store reply
** Returned value:		1 if command correct, 0 if failed
**
*****************************************************************************/

uint8_t wiflySendHTTP ( char *comand , char *resposta ){

	char text [400];
	char str_ln[3] = { 0x0D , 0x0A , 0x00 }; /* we define line feed */
	uint8_t i, correcte = 0;
	char *reply;

	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */
	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	memset ( text , 0x00 , sizeof ( text ) );   // clean buff text
	//UART0Count = 0;

	/* We prepare commands to be sent to Wifly */

	strcat ( text , "set ip proto 18" );
	strcat ( text , str_ln );
	strcat ( text , "set dns name www.arpalab.appspot.com" );
	strcat ( text , str_ln );
	strcat ( text , "set ip host 0" );
	strcat ( text , str_ln );
	strcat ( text , "set ip remote 80" );
	strcat ( text , str_ln );
	strcat ( text , "set com remote 0" );
	strcat ( text , str_ln );
	strcat ( text , "set com idle 6" );
	strcat ( text , str_ln );
	strcat ( text , "set option format 0" );
	strcat ( text , str_ln );
	strcat ( text , "open" );
	strcat ( text , str_ln );

	/* We send commands to wifly */

	printText ( 0 , text );
	passiveDelayMs ( 5000 );

	/* Now we prepare order to be sent to server */

	memset ( text , 0x00 , sizeof ( text ) );
	strcat ( text , comand );
	strcat ( text , str_ln );
	strcat ( text , "Host: arpalab.appspot.com" );
	strcat ( text , str_ln );
	strcat ( text , str_ln );

	/* We send order to server */

	printText ( 0 ,text );

	/* We receive the reply from the server */
	memset ( text , 0x00 , sizeof ( text ) );
	UARTScan ( text , 8000 );

	/* We check the answer */

	if ( parseString ( text , MSG_GET ) != 0 ){
				correcte = 1;
			}

	i = parseString ( text , "chunked" );

	reply =  &( text[i+7] );
	cutString ( reply , '\r' );
	memset ( resposta , 0x00 , sizeof ( resposta ) );
	strcpy ( resposta , reply );

	/* We return to command mode after connection is closed */

	passiveDelayMs ( 2000 );
	printText ( 0 , str_ln );
	printText ( 0 , str_ln );
	printText ( 0 , "$$$" );
	passiveDelayMs ( 1000 );

	}

	xSemaphoreGive ( xMutex );

	return correcte;

}

/*****************************************************************************
** Function name:		wiflyGetTxPower
**
** Descriptions:		Get the transmission power of Wifly
**
** parameters:			string to store returned value
** Returned value:		1 if command correct, 0 if failed
**
*****************************************************************************/

uint8_t wiflyGetTxPower ( char *resposta ){

	char respostaWifly[BUFSIZE];
	char *reply;
	uint32_t i;

	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */
	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	/* We send the command to Wifly */
	printText ( 0 , "get wlan\r\n" );

	/* We receive the answer from Wifly */
	UARTScan ( respostaWifly , 4000 );

	/* we check the answer from wifly */

	i = parseString ( respostaWifly , "TxPower=" );

	if ( i != 0){

			reply = &( respostaWifly[i] );
			cutString ( reply , '\r' );
			strcpy ( resposta , reply );

			i = 1;
	}
	}

	xSemaphoreGive ( xMutex );
	return i;

}

/*****************************************************************************
** Function name:		wiflySetTxPower
**
** Descriptions:		Set the transmission power of Wifly
**
** parameters:			New Value of new transmission power
** Returned value:		1 if comnand correct, 0 if failed
**
*****************************************************************************/

uint8_t wiflySetTxPower( uint8_t power ){

	char texte[BUFSIZE];
	char respostaWifly[BUFSIZE];
	uint8_t correcte =0 ;

	/* Attempt to take the semaphore, blocking indefinitely if the mutex is not
	available immediately. */
	xSemaphoreTake ( xMutex, portMAX_DELAY );
	{

	sprintf ( texte , "set wlan tx %c\r\n" , power+48 );

	/* We send the command to Wifly */

	printText ( 0 , texte );

	/* We receive the answer from Wifly */
	UARTScan ( respostaWifly , 10000 );

	/* we check the answer from wifly */

	if ( parseString ( respostaWifly , "AOK" ) != 0 ){
		correcte = 1;
	}

	}

	xSemaphoreGive ( xMutex );

	return correcte;
}

/*****************************************************************************
**                            End Of File
******************************************************************************/
